/*
   Author:  Rony G. Flatscher
   Purpose: Demonstrate another possibility to script ODF using ooRexx with the
            Java camouflaging support of BSF4ooRexx (makes Java typeless and look
            like ooRexx)

   Inspired by Marco Fioretti's presentation at the 2010 OpenOffice Conference in Budapest,
            cf. <http://www.ooocon.org/index.php/ooocon/2010>

   Needs:   - ooRexx (FOSS):
              - Linux: a build of ooRexx with a revision >= 6133, from <http://build.oorexx.org>
              - Windows: <http://www.oorexx.org/download.html>

            - BSF4ooRexx (FOSS):
              - <http://wi.wu-wien.ac.at/rgf/rexx/bsf4oorexx/current/>
              - read the "readme*.txt" file of your platform for installation instructions

            - the Java odftoolkit 0.8.6 as of July 2010, downloaded from:
              <http://odftoolkit.org/projects/odfdom/downloads/download/current-version%252Fodfdom-0.8.6-binaries.zip>

              directions:
                unzip -j odfdom-0.8.6-binaries.zip odfdom-0.8.6-binaries/odfdom.jar
                ... will extract the "odfdom.jar" Java archive (a zip archive), which needs
                    to get added to the CLASSPATH environment variable for Java to find the
                    contained classes

                Linux:
                   - export CLASSPATH=`pwd`/odfdom.jar:$CLASSPATH

                Windows:
                   - set CLASSPATH=%cd%\odfdom.jar;%CLASSPATH%

            - the Java XSLT processor "Xerces" from the Apache Software foundation, downloaded from:
              <http://www.ecoficial.com/apachemirror//xerces/j/Xerces-J-bin.2.10.0.zip>

              directions:
                unzip -j Xerces-J-bin.2.10.0.zip xerces-2_10_0/xercesImpl.jar
                ... will extract the "xercesImpl.jar" Java archive (a zip archive), which needs
                    to get added to the CLASSPATH environment variable for Java to find the
                    contained classes

                Linux:
                   - export CLASSPATH=`pwd`/xercesImpl.jar:$CLASSPATH

                Windows:
                   - set CLASSPATH=%cd%\xercesImpl.jar;%CLASSPATH%

   To run:  rexx test4marco-fioretti.rex test4marco-fioretti.odt test4marco-fioretti.txt
*/

parse arg odtFilename customerFilename . -- get command line argument, make sure last argument is stripped as well

   -- load the Java class "OdfPackage" and use it for manipulating the ODF-files
odfPackageClz=bsf.loadClass("org.odftoolkit.odfdom.pkg.OdfPackage")  -- import the Java class object
odfPackage   =odfPackageClz~loadPackage(odtFileName)  -- create and fetch a package object

   -- read content.xml file
packagePath="content.xml"              -- file from the package
mediaType  =odfPackage~getFileEntry(packagePath)~getMediaType  -- needed for save operation later
bytes=odfPackage~getBytes(packagePath) -- get stream as a byte array (as raw bytes)
rexxString=BsfRawBytes(bytes)          -- convert Java byte array to Rexx string of raw bytes/octets

   -- read and parse customer file
a=readCustomerFile(customerFileName)   -- returns an array of directories, each representing a parsed customer

   -- get and set todays date
date=.dateTime~new~europeanDate("/")   -- get current date in dd/mm/yy (use "/" as a delimiter)

   -- create as many odt-files as we have customers in the txt-file
loop i=1 to a~items        -- loop over array
   newString=changeValues(rexxString,a[i],date) -- carry out the changes
   newBytes=BsfRawBytes(newString)        -- convert Rexx string to Java byte array
   odfPackage~insert(newBytes,packagePath,mediaType)  -- write back data (replaces old content!)
   odfPackage~save("new_"i"_"odtFileName) -- save under a new name
end
odfPackage~close                       -- close the package object, we are done


::requires BSF.CLS   -- load the ooRexx Java support (make Java typeless and look like ooRexx)


::routine changeValues        -- carries out the replacements in the content.xml text
  use arg string, dir, date

  mb=.MutableBuffer~new(string)  -- use the ooRexx MutableBuffer in case we have plenty of string changes
  mb~changeStr("MF_DATE",    date)
  mb~changeStr("MF_NAME",    dir~MF_NAME~strip)
  mb~changeStr("MF_STREET",  dir~MF_STREET~strip)
  mb~changeStr("MF_ZIP",     dir~MF_ZIP~strip)
  mb~changeStr("MF_CITY",    dir~MF_CITY~strip)
  mb~changeStr("MF_COUNTRY", dir~MF_COUNTRY~strip)
  return mb~string               -- return a plain string


::routine readCustomerFile    -- reads and parses a customer file
  use arg filename

  deli="/;\"      -- delimiter string for columns
  a=.array~new    -- array to collect the parsed lines from the file
  loop while chars(filename)>0   -- as long as there are unread characters loop
     line=linein(filename)
     if line="" | left(line,1)=";" then iterate    -- a comment or empty line, ignore it
     d=.directory~new   -- directory to contain the parsed values of the individual lines
     parse var line d~mf_name (deli) d~mf_street (deli) d~mf_zip (deli) d~mf_city (deli) d~mf_country
     a~append(d)  -- save directory object in array
  end
  return a        -- return array of directory objects, each representing a customer

